package de.hdm_stuttgart.mi.sd1.task1;

/**
 * <p>Handle web addresses.</p>
 */
public class C_WebUrl {

    /**
     * <p>Accessing web sites e.g. <code>www.wikipedia.org</code> requires entering the address in a web browser:</p>
     *
     * <img style="outline: 2px solid;"
     *     src="doc-files/weburlBlank.png" alt="A web address without protocol specification" >
     *
     * <p>Entering <code>www.wikipedia.org</code> fulfills its purpose. However behind the scenes either a
     * <code>http://</code> or <code>https://</code> protocol specification is being silently prepended:</p>
     *
     * <img style="outline: 2px solid;"
     *     src="doc-files/weburlProtocol.png" alt="A web address including a protocol specification">
     *
     * <p>The current method adds a missing protocol specification according to the following rules:</p>
     *
     * <ul>
     *     <li>An address already starting with either <code>http://</code> or <code>https://</code> is just being
     *      copied.</li>
     *
     *     <li>
     *        <p>An address not starting with either <code>http://</code> or <code>https://</code> receives a
     *             protocol prefix according to the <code>tlsSupport</code> parameter.</p>
     *     </li>
     *  </ul>
     *  <p>Some examples:</p>
     *  <table class="goikTableDefaults">
     *     <caption>Optional protocol prefix examples for <code>www.wikipedia.org</code></caption>
     *     <tr>
     *         <th><code>address</code></th>
     *         <th><code>tlsSupport</code></th>
     *         <th>Return value</th>
     *     </tr>
     *     <tr>
     *         <td style="color:blue"><code>www.wikipedia.org</code></td>
     *         <td><code style="color:red">true</code></td>
     *         <td><code style="color:red">https</code>://<code style="color:blue">www.wikipedia.org</code></td>
     *     </tr>
     *     <tr>
     *         <td style="color:blue"><code>www.wikipedia.org</code></td>
     *         <td><code style="color:red">false</code></td>
     *         <td><code style="color:red">http</code>://<code style="color:blue">www.wikipedia.org</code></td>
     *     </tr>
     *     <tr>
     *         <td style="color:blue"><code>http://www.wikipedia.org</code></td>
     *         <td>Protocol specification <code>http://</code> present, value irrelevant</td>
     *         <td style="color:blue"><code>http://www.wikipedia.org</code></td>
     *     </tr>
     *     <tr>
     *         <td style="color:blue"><code>https://www.wikipedia.org</code></td>
     *         <td>Protocol specification <code>https://</code> present, value irrelevant</td>
     *         <td style="color:blue"><code>https://www.wikipedia.org</code></td>
     *     </tr>
     * </table>
     *
     * @param address A website's address. May either be a raw name or a full http/https prefixed protocol address.
     * @param tlsSupport If <code>true</code> the server supports
     *                   <code><span style="color:red">https</span>://...</code> connections. Otherwise, only
     *                   <code><span style="color:red">http</span>://...</code> is being supported.
     *
     * @return A fully <code>http://</code> or <code>https://</code> prefixed protocol address.
     *
     * <section class="implementationHints">
     *     <h4>Hint:</h4>
     *     
     *     <p>{@link String#startsWith(String)}</p>
     * </section>
     */
    public static String amendWebAddress(final String address, final boolean tlsSupport) {
        if (address.startsWith("http://") | address.startsWith("https://")) {
            return address;
        } else if (tlsSupport){
            return "https://" + address;
        } else {
            return "http://" + address;
        }
    }
    private C_WebUrl(){/* Ignore me: My sole purpose is suppressing default constructor javadoc generation */}
}
