package de.hdm_stuttgart.mi.sd1.task1;

/**
 * Array helper method.
 */
public class D_Array {

    /**
     * The special value -1 indicating two arrays are not being related by any sequence of right cyclic rotations.
     * Example: {1, 2} and {1, 3}.
     */
    static public final int NOT_CYCLICALLY_RELATED = -1 ;

    /**
     * <p>Part 2: Check two arrays for being related by an integer number of successive right cyclic rotations or not.</p>
     *
     * <p>Consider a right cyclic array rotating example:</p>
     *
     * <pre>
     *      <span style="color:red">⬐─────┐</span>
     *     {1, 7, <span style="color:blue">3</span>} ⇨ {<span style="color:blue">3</span>, 1, 7}
     *       <span style="color:red">⤻  ⤻</span>
     * </pre>
     *
     * <p>The arrays <code>{1, 7, 3}</code> and <code>{3, 1, 7}</code> are thus being related by one right
     * cyclic rotation.</p>
     *
     * <p>Arrays may also be related by a succession of multiple right cyclic rotations. Example:</p>
     *
     * <p><code>{1, 8, 3, 4}</code> and <code>{3, 4, 1, 8}</code> are being related by a sequence of two right cyclic
     * rotations:</p>
     *
     * <pre>
     * <span style="color:red">     ⬐────────┐    ⬐────────┐</span>
     *     {1, 8, 3, <span style="color:blue">4</span>} ⇨ {<span style="color:blue">4</span>, 1, 8, <span
     *     style="color:orange">3</span>} ⇨ {<span style="color:orange">3</span>, <span style="color:blue">4</span>, 1, 8}
     * <span style="color:red">      ⤻  ⤻ ⤻       ⤻  ⤻ ⤻</span>
     * </pre>
     *
     * <p>In this case a value of 2 rotations is being returned.</p>
     *
     * <h4>Special cases: </h4>
     *
     * <ul>
     *     <li>Two identical arrays are being related by zero rotations.</li>
     *     <li>Two arrays of different size are always unrelated / {@link #NOT_CYCLICALLY_RELATED} .</li>
     * </ul>
     *
     * @param array1 The first array. Values must remain untouched.
     * @param array2 The second array. Values must remain untouched.
     *
     * @return Either the number of right cyclic rotations or the special value {@link #NOT_CYCLICALLY_RELATED} if both
     * arrays are not being related by a sequence of cyclic rotations.
     */
    static public int isRotatedBy(final int[] array1, int[] array2) {
        final int length = array1.length;
        if (length == array2.length) {
            if (0 == length) {
                return 0;
            }
            for (int rotation = 0; rotation < length; rotation++) {
                boolean allValuesEqual = true;
                for (int i = 0; i < length; i++) {
                    if (array1[i] != array2[(i + rotation) % length]) {
                        allValuesEqual = false;
                        break;
                    }
                }
                if (allValuesEqual) {
                    return rotation;
                }
            }
        }
        return NOT_CYCLICALLY_RELATED; // No rotation found
    }
    private D_Array(){/* Ignore me, suppressing implicit default constructor Javadoc HTML generation */}
}
