package de.hdm_stuttgart.mi.sd1.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import java.util.Arrays;

import static de.hdm_stuttgart.mi.sd1.task1.D_Array.NOT_CYCLICALLY_RELATED;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class D_ArrayTest extends ExaminationTestDefaults {

    private static void assertRelatedByRotation(final int expectedRotation, final int[] array1, int[] array2) {

        final int[] array1Copy = Arrays.copyOf(array1, array1.length),
                array2Copy = Arrays.copyOf(array2, array2.length);

        final String errorMessage;
        if (NOT_CYCLICALLY_RELATED == expectedRotation) {
            errorMessage = Arrays.toString(array1) + " and " + Arrays.toString(array2) +
                    " are not being cyclically related.";
        } else {
            errorMessage = Arrays.toString(array1) + " and " + Arrays.toString(array2) +
                    " are being cyclically related by " + expectedRotation + " right rotation(s).";
        }
        Assert.assertEquals(errorMessage, expectedRotation, D_Array.isRotatedBy(array1, array2));

        Assert.assertArrayEquals("You must not change values in array1 parameter", array1Copy, array1);
        Assert.assertArrayEquals("You must not change values in array2 parameter", array2Copy, array2);
    }

    @Test
    public void test010_Minimum() {
        assertRelatedByRotation(0, new int[]{}, new int[]{});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED, new int[]{}, new int[]{1});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED, new int[]{3}, new int[]{});
    }
    @Test
    public void test020One() {
        assertRelatedByRotation(0,  new int[]{77}, new int[]{77});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{8}, new int[]{77});
    }

    @Test
    @Marking(points = 2)
    public void test030Two() {
        assertRelatedByRotation(0,  new int[]{-1, 5}, new int[]{-1, 5});
        assertRelatedByRotation(1,  new int[]{-1, 5}, new int[]{5, -1});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 5}, new int[]{-5, 1});
    }
    @Test
    @Marking(points = 2)
    public void test040Three() {
        assertRelatedByRotation( 0,  new int[]{2, -6, 9}, new int[]{ 2, -6,  9});
        assertRelatedByRotation( 1,  new int[]{2, -6, 9}, new int[]{ 9,  2, -6});
        assertRelatedByRotation( 2,  new int[]{2, -6, 9}, new int[]{-6,  9,  2});

        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{2, -6, 9}, new int[]{ 2,  9, -6});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{2, -6, 9}, new int[]{-6,  2,  9});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{2, -6, 9}, new int[]{-6,  2});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 2, 4}, new int[]{-1, 2, 5});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 2, 4}, new int[]{-2, 2, 4});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 2, 4}, new int[]{-1, 3, 4});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 2, 4}, new int[]{-1, 3, 4, 3});
        assertRelatedByRotation(NOT_CYCLICALLY_RELATED,  new int[]{-1, 2, 4, 1}, new int[]{-1, 3, 4});
    }
    @Test
    @Marking(points = 2)
    public void test050Twenty() {
        assertRelatedByRotation( 0,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10});

        assertRelatedByRotation( 1,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{10, 1, 2, 3, 4, 5, 6, 7, 8, 9});

        assertRelatedByRotation( 2,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{9, 10, 1, 2, 3, 4, 5, 6, 7, 8});

        assertRelatedByRotation( 3,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{8, 9, 10, 1, 2, 3, 4, 5, 6, 7});

        assertRelatedByRotation( 4,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{7, 8, 9, 10, 1, 2, 3, 4, 5, 6});

        assertRelatedByRotation( 5,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{6, 7, 8, 9, 10, 1, 2, 3, 4, 5});

        assertRelatedByRotation( 6,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{5, 6, 7, 8, 9, 10, 1, 2, 3, 4});

        assertRelatedByRotation( 7,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{4, 5, 6, 7, 8, 9, 10, 1, 2, 3});

        assertRelatedByRotation( 8,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{3, 4, 5, 6, 7, 8, 9, 10, 1, 2});

        assertRelatedByRotation( 9,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                          */     new int[]{2, 3, 4, 5, 6, 7, 8, 9, 10, 1});

        assertRelatedByRotation( NOT_CYCLICALLY_RELATED,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                                 */     new int[]{1, 2, 3, 4, 6, 5, 7, 8, 9, 10});

        assertRelatedByRotation( NOT_CYCLICALLY_RELATED,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                                 */     new int[]{1, 2, 3, 4, 6, 5, 7, 8, 9});

        assertRelatedByRotation( NOT_CYCLICALLY_RELATED,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9},
                /*                                 */     new int[]{1, 2, 3, 4, 6, 5, 7, 8, 9, 10});

        assertRelatedByRotation( NOT_CYCLICALLY_RELATED,  new int[]{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
                /*                                 */     new int[]{});


    }
}
